"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const https_1 = __importDefault(require("https"));
const url_1 = require("url");
const passport_strategy_1 = require("passport-strategy");
class Strategy extends passport_strategy_1.Strategy {
    constructor(options, verify) {
        super();
        if (!verify) {
            throw new TypeError("NotionStrategy requires a verify callback");
        }
        if (!options.clientID) {
            throw new TypeError("NotionStrategy requires a clientID");
        }
        if (!options.clientSecret) {
            throw new TypeError("NotionStrategy requires a clientSecret");
        }
        if (!options.callbackURL) {
            throw new TypeError("NotionStrategy require an Callback URL option");
        }
        this.name = "notion";
        this._verify = verify;
        this._options = options;
        this._clientSecret = options.clientSecret;
        this._clientID = options.clientID;
        this._tokenURL = options.tokenURL || "https://api.notion.com/v1/oauth/token";
        this._authorizationURL = options.authorizationURL || "https://api.notion.com/v1/oauth/authorize";
    }
    async authenticate(req, options) {
        var _a;
        options = options || {};
        if (req.query && req.query.code) {
            try {
                const oauthData = await this.getOAuthAccessToken(req.query.code);
                if (oauthData.owner.type !== "user") {
                    throw new Error(`Notion API token not owned by user, instead: ${oauthData.owner.type}`);
                }
                this._verify(req, oauthData.access_token, undefined, oauthData, oauthData.owner.user, (err, user, info) => {
                    if (err)
                        return this.error(err);
                    if (!user)
                        return this.fail(info /* ??? */);
                    this.success(user);
                });
            }
            catch (error) {
                this.error(error);
            }
        }
        else {
            const authUrl = new url_1.URL(this._authorizationURL);
            authUrl.searchParams.set("client_id", this._clientID);
            authUrl.searchParams.set("redirect_uri", this._options.callbackURL);
            authUrl.searchParams.set("response_type", "code");
            if ((_a = this._options) === null || _a === void 0 ? void 0 : _a.state) {
                authUrl.searchParams.set("state", this._options.state);
            }
            const location = authUrl.toString();
            this.redirect(location);
        }
    }
    async getOAuthAccessToken(code) {
        let accessTokenURLObject = new url_1.URL(this._tokenURL);
        const accessTokenBody = {
            grant_type: "authorization_code",
            code,
            redirect_uri: this._options.callbackURL,
        };
        const encodedCredential = Buffer.from(`${this._clientID}:${this._clientSecret}`).toString("base64");
        const requestOptions = {
            hostname: accessTokenURLObject.hostname,
            path: accessTokenURLObject.pathname,
            headers: {
                Authorization: `Basic ${encodedCredential}`,
                "Content-Type": "application/json",
            },
            method: "POST",
        };
        return new Promise((resolve, reject) => {
            const accessTokenRequest = https_1.default.request(requestOptions, (res) => {
                let data = "";
                res.on("data", (d) => {
                    data += d;
                });
                res.on("end", () => {
                    try {
                        resolve(JSON.parse(data));
                    }
                    catch (error) {
                        reject(error);
                    }
                });
            });
            accessTokenRequest.on("error", reject);
            accessTokenRequest.write(JSON.stringify(accessTokenBody));
            accessTokenRequest.end();
        });
    }
}
exports.default = Strategy;
//# sourceMappingURL=strategy.js.map